package net.sf.sido.spring.mvc;

import net.sf.sido.DataFactory;
import net.sf.sido.DataObject;
import net.sf.sido.pojo.DOFactory;
import net.sf.sido.pojo.DOObject;
import org.apache.commons.lang.reflect.MethodUtils;
import org.springframework.core.MethodParameter;
import org.springframework.web.context.request.NativeWebRequest;

public class DOObjectArgumentResolver extends AbstractSiDOArgumentResolver {

    /**
     * Always validate the parameter
     */
    @Override
    protected void check(DataObject o, MethodParameter methodParameter) {
        validate(o, methodParameter);
    }

    @Override
    protected String getExpectedDataTypeName(MethodParameter methodParameter, NativeWebRequest webRequest, boolean typeRequired) throws Exception {
        return (String) MethodUtils.invokeStaticMethod(methodParameter.getParameterType(), "getQualifiedTypeName", new Object[0]);
    }

    @Override
    protected DataFactory getDataFactory() {
        return DOFactory.getContextFactory().getDataFactory();
    }

    @Override
    protected boolean supportsParameterType(Class<?> parameterType) {
        return DOObject.class.isAssignableFrom(parameterType);
    }

    /**
     * No initialisation, only validation
     *
     * @see #check(net.sf.sido.DataObject, org.springframework.core.MethodParameter)
     */
    @Override
    protected void init(DataObject o, MethodParameter methodParameter) {
    }

    @Override
    protected Object transform(DataObject dataObject, MethodParameter methodParameter, NativeWebRequest webRequest) {
        DOObject o = DOFactory.getContextFactory().create(dataObject);
        if (methodParameter.getParameterType().isInstance(o)) {
            return o;
        } else {
            throw new IllegalStateException(String.format("Parameter class, %s, is not of the expected type %s",
                    o.getClass().getName(),
                    methodParameter.getParameterType().getName()));
        }
    }
}
