package net.sf.sido.spring.mvc;

import net.sf.sido.DataFactory;
import net.sf.sido.DataObject;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.MethodParameter;
import org.springframework.web.context.request.NativeWebRequest;

public class DataObjectArgumentResolver extends AbstractSiDOArgumentResolver {

    public static final String REQUEST_PARAMETER_TYPE = "sido:type";

    private final DataFactory dataFactory;

    @Autowired
    public DataObjectArgumentResolver(DataFactory dataFactory) {
        this.dataFactory = dataFactory;
    }

    protected void check(DataObject o, MethodParameter methodParameter) {
        HasDataType hasDataType = methodParameter.getParameterAnnotation(HasDataType.class);
        if (hasDataType != null && hasDataType.validate()) {
            validate(o, methodParameter);
        }
    }

    @Override
    protected DataFactory getDataFactory() {
        return dataFactory;
    }

    @Override
    protected String getExpectedDataTypeName(MethodParameter methodParameter, NativeWebRequest webRequest, boolean typeRequired) throws Exception {
        // Gets the DataType annotation
        HasDataType hasDataType = methodParameter.getParameterAnnotation(HasDataType.class);
        if (hasDataType != null) {
            return hasDataType.value();
        } else {
            String typeName = webRequest.getParameter(REQUEST_PARAMETER_TYPE);
            if (StringUtils.isBlank(typeName)) {
                if (typeRequired) {
                    throw new IllegalStateException(String.format(
                            "Type for DataObject %s is required. You can use a HasDataType annotation on the parameter or add a %s parameter in the request.",
                            methodParameter.getParameterName(),
                            REQUEST_PARAMETER_TYPE));
                } else {
                    return null;
                }
            } else {
                return typeName;
            }
        }
    }

    @Override
    protected void init(DataObject o, MethodParameter methodParameter) {
        HasDataType hasDataType = methodParameter.getParameterAnnotation(HasDataType.class);
        if (hasDataType != null && hasDataType.init()) {
            o.init(true);
        }
    }

    protected boolean supportsParameterType(Class<?> parameterType) {
        return DataObject.class.isAssignableFrom(parameterType);
    }

    @Override
    protected Object transform(DataObject dataObject, MethodParameter methodParameter, NativeWebRequest webRequest) {
        return dataObject;
    }
}
